@@ -16,6 +16,7 @@
 
 #if defined(CONFIG_PROC_FS)
 
+#include "nv-procfs.h"
 #include "nv_compiler.h"
 #include "nv-reg.h"
 #include "conftest/patches.h"
@@ -23,6 +24,11 @@
 #include "nv-instance.h"
 #include "nv-ibmnpu.h"
 
+#define NV_DEFINE_SINGLE_NVRM_PROCFS_FILE(name) \
+    NV_DEFINE_SINGLE_PROCFS_FILE(name, \
+                                 NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE, \
+                                 NV_READ_UNLOCK_SYSTEM_PM_LOCK)
+
 static const char *__README_warning = \
     "The NVIDIA graphics driver tries to detect potential problems\n"
     "with the host system and warns about them using the system's\n"
@@ -46,9 +52,14 @@ extern char *NVreg_RegistryDwordsPerDevice;
 extern char *NVreg_RmMsg;
 extern char *NVreg_AssignGpus;
 extern char *NVreg_GpuBlacklist;
+extern char *NVreg_TemporaryFilePath;
 
 static char nv_registry_keys[NV_MAX_REGISTRY_KEYS_LENGTH];
 
+#if defined(CONFIG_PM)
+static nv_pm_action_depth_t nv_pm_action_depth = NV_PM_ACTION_DEPTH_DEFAULT;
+#endif
+
 static int nv_procfs_read_registry(struct seq_file *s, void *v);
 
 #define NV_NUMA_STATUS_MSG_LEN      (32)
@@ -76,7 +87,7 @@ nv_procfs_read_gpu_info(
 {
     nv_state_t *nv = s->private;
     nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
-    struct pci_dev *dev = nvl->dev;
+    struct pci_dev *pci_dev = nvl->pci_dev;
     char *type, *fmt, tmpstr[NV_DEVICE_NAME_LENGTH];
     int status;
     NvU8 *uuid;
@@ -91,7 +102,7 @@ nv_procfs_read_gpu_info(
 
     if (NV_IS_GVI_DEVICE(nv))
     {
-        if (rm_gvi_get_device_name(sp, nv, dev->device, NV_DEVICE_NAME_LENGTH,
+        if (rm_gvi_get_device_name(sp, nv, pci_dev->device, NV_DEVICE_NAME_LENGTH,
                                    tmpstr) != NV_OK)
         {
             strcpy (tmpstr, "Unknown");
@@ -99,8 +110,8 @@ nv_procfs_read_gpu_info(
     }
     else
     {
-        if (rm_get_device_name(sp, nv, dev->device, dev->subsystem_vendor,
-                    dev->subsystem_device, NV_DEVICE_NAME_LENGTH,
+        if (rm_get_device_name(sp, nv, pci_dev->device, pci_dev->subsystem_vendor,
+                    pci_dev->subsystem_device, NV_DEVICE_NAME_LENGTH,
                     tmpstr) != NV_OK)
         {
             strcpy (tmpstr, "Unknown");
@@ -144,18 +155,18 @@ nv_procfs_read_gpu_info(
         }
     }
 
-    if (nv_find_pci_capability(dev, PCI_CAP_ID_EXP))
+    if (nv_find_pci_capability(pci_dev, PCI_CAP_ID_EXP))
         type = "PCIe";
     else
         type = "PCI";
     seq_printf(s, "Bus Type: \t %s\n", type);
 
     seq_printf(s, "DMA Size: \t %d bits\n",
-     nv_count_bits(dev->dma_mask));
-    seq_printf(s, "DMA Mask: \t 0x%llx\n", dev->dma_mask);
+     nv_count_bits(pci_dev->dma_mask));
+    seq_printf(s, "DMA Mask: \t 0x%llx\n", pci_dev->dma_mask);
     seq_printf(s, "Bus Location: \t %04x:%02x:%02x.%x\n",
                    nv->pci_info.domain, nv->pci_info.bus,
-                   nv->pci_info.slot, PCI_FUNC(dev->devfn));
+                   nv->pci_info.slot, PCI_FUNC(pci_dev->devfn));
     seq_printf(s, "Device Minor: \t %u\n", nvl->minor_num);
 
 #if defined(DEBUG)
@@ -178,7 +189,7 @@ nv_procfs_read_gpu_info(
     return 0;
 }
 
-NV_DEFINE_PROCFS_SINGLE_FILE(gpu_info);
+NV_DEFINE_SINGLE_NVRM_PROCFS_FILE(gpu_info);
 
 static int
 nv_procfs_read_version(
@@ -192,7 +203,28 @@ nv_procfs_read_version(
     return 0;
 }
 
-NV_DEFINE_PROCFS_SINGLE_FILE(version);
+NV_DEFINE_SINGLE_NVRM_PROCFS_FILE(version);
+
+static void
+nv_procfs_close_file(
+    nv_file_private_t *nvfp
+)
+{
+    nvidia_stack_t *sp;
+
+    if (nvfp->data != NULL)
+    {
+        os_free_mem(nvfp->data);
+    }
+
+    sp  = nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
+    if (sp != NULL)
+    {
+        nv_kmem_cache_free_stack(sp);
+    }
+
+    nv_free_file_private(nvfp);
+}
 
 static int
 nv_procfs_open_file(
@@ -201,9 +233,10 @@ nv_procfs_open_file(
     nv_file_private_t **nvfpp
 )
 {
+    int retval = 0;
+    NV_STATUS status;
     nv_file_private_t *nvfp = NULL;
     nvidia_stack_t *sp = NULL;
-    int status;
 
     nvfp = nv_alloc_file_private();
     if (nvfp == NULL)
@@ -217,25 +250,31 @@ nv_procfs_open_file(
     if (0 == (file->f_mode & FMODE_WRITE))
         goto done;
 
-    status = nv_kmem_cache_alloc_stack(&sp);
-    if (status != 0)
+    retval = nv_kmem_cache_alloc_stack(&sp);
+    if (retval != 0)
     {
-        nv_free_file_private(nvfp);
-        return status;
+        goto done;
     }
 
-    if (NV_OK != os_alloc_mem((void **)&nvfp->data, NV_PROC_WRITE_BUFFER_SIZE))
+    status = os_alloc_mem((void **)&nvfp->data, NV_PROC_WRITE_BUFFER_SIZE);
+    if (status != NV_OK)
     {
-        nv_free_file_private(nvfp);
-        nv_kmem_cache_free_stack(sp);
-        return -ENOMEM;
+        retval = -ENOMEM;
+        goto done;
     }
 
     os_mem_set((void *)nvfp->data, 0, NV_PROC_WRITE_BUFFER_SIZE);
     nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS] = sp;
 
 done:
+    if (retval < 0)
+    {
+        nv_procfs_close_file(nvfp);
+        return retval;
+    }
+
     *nvfpp = nvfp;
+
     return 0;
 }
 
@@ -249,9 +288,23 @@ nv_procfs_open_registry(
     int retval;
 
     retval = nv_procfs_open_file(inode, file, &nvfp);
-    if (retval == 0)
+    if (retval < 0)
     {
-        single_open(file, nv_procfs_read_registry, nvfp);
+        return retval;
+    }
+
+    retval = single_open(file, nv_procfs_read_registry, nvfp);
+    if (retval < 0)
+    {
+        nv_procfs_close_file(nvfp);
+        return retval;
+    }
+
+    retval = NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE();
+    if (retval < 0)
+    {
+        single_release(inode, file);
+        nv_procfs_close_file(nvfp);
     }
 
     return retval;
@@ -264,21 +317,16 @@ nv_procfs_close_registry(
 )
 {
     struct seq_file *s = file->private_data;
-    nv_file_private_t *nvfp;
+    nv_file_private_t *nvfp = s->private;
     nv_state_t *nv;
     nv_linux_state_t *nvl = NULL;
-    nvidia_stack_t *sp = NULL;
+    nvidia_stack_t *sp = nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
     char *key_name, *key_value, *registry_keys;
     size_t key_len, len;
     long count;
     NV_STATUS rm_status;
     int rc = 0;
 
-    nvfp = s->private;
-    single_release(inode, file);
-
-    sp = nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
-
     if (0 != nvfp->off)
     {
         nv = nvfp->proc_data;
@@ -326,16 +374,11 @@ nv_procfs_close_registry(
     }
 
 done:
-    if (NULL != nvfp->data)
-        os_free_mem(nvfp->data);
+    NV_READ_UNLOCK_SYSTEM_PM_LOCK();
 
-    if (sp != NULL)
-    {
-        nv_kmem_cache_free_stack(sp);
-        nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS] = NULL;
-    }
+    single_release(inode, file);
 
-    nv_free_file_private(nvfp);
+    nv_procfs_close_file(nvfp);
 
     return rc;
 }
@@ -362,10 +405,13 @@ nv_procfs_read_params(
                (NVreg_AssignGpus != NULL) ? NVreg_AssignGpus : "");
     seq_printf(s, "GpuBlacklist: \"%s\"\n",
                (NVreg_GpuBlacklist != NULL) ? NVreg_GpuBlacklist : "");
+    seq_printf(s, "TemporaryFilePath: \"%s\"\n",
+               (NVreg_TemporaryFilePath != NULL) ? NVreg_TemporaryFilePath : "");
+
     return 0;
 }
 
-NV_DEFINE_PROCFS_SINGLE_FILE(params);
+NV_DEFINE_SINGLE_NVRM_PROCFS_FILE(params);
 
 static int
 nv_procfs_read_registry(
@@ -445,6 +491,168 @@ static struct file_operations nv_procfs_registry_fops = {
     .release = nv_procfs_close_registry,
 };
 
+#if defined(CONFIG_PM)
+static int
+nv_procfs_show_suspend_depth(
+    struct seq_file *m,
+    void *v
+)
+{
+    seq_printf(m, "default modeset uvm\n");
+    return 0;
+}
+
+static ssize_t
+nv_procfs_write_suspend_depth(
+    struct file *file,
+    const char __user *buf,
+    size_t count,
+    loff_t *pos
+)
+{
+    char kbuf[sizeof("modeset\n")];
+    unsigned i;
+
+    if (!NV_IS_SUSER())
+    {
+        return -EPERM;
+    }
+
+    if (count < strlen("uvm") || count > sizeof(kbuf))
+    {
+        return -EINVAL;
+    }
+
+    if (copy_from_user(kbuf, buf, count))
+    {
+        return -EFAULT;
+    }
+
+    count = min(count, sizeof(kbuf) - 1);
+    for (i = 0; i < count && isalpha(kbuf[i]); i++);
+    kbuf[i] = '\0';
+
+    if (strcasecmp(kbuf, "uvm") == 0)
+    {
+        nv_pm_action_depth = NV_PM_ACTION_DEPTH_UVM;
+    }
+    else if (strcasecmp(kbuf, "modeset") == 0)
+    {
+        nv_pm_action_depth = NV_PM_ACTION_DEPTH_MODESET;
+    }
+    else if (strcasecmp(kbuf, "default") == 0)
+    {
+        nv_pm_action_depth = NV_PM_ACTION_DEPTH_DEFAULT;
+    }
+    else
+    {
+        return -EINVAL;
+    }
+
+    return count;
+}
+
+static int
+nv_procfs_open_suspend_depth(
+    struct inode *inode,
+    struct file *file
+)
+{
+    return single_open(file, nv_procfs_show_suspend_depth, NULL);
+}
+
+static struct file_operations nv_procfs_suspend_depth_fops = {
+    .owner   = THIS_MODULE,
+    .open    = nv_procfs_open_suspend_depth,
+    .read    = seq_read,
+    .write   = nv_procfs_write_suspend_depth,
+    .llseek  = seq_lseek,
+    .release = single_release
+};
+
+static int
+nv_procfs_show_suspend(
+    struct seq_file *m,
+    void *v
+)
+{
+    seq_printf(m, "suspend hibernate resume\n");
+    return 0;
+}
+
+static ssize_t
+nv_procfs_write_suspend(
+    struct file *file,
+    const char __user *buf,
+    size_t count,
+    loff_t *pos
+)
+{
+    NV_STATUS status;
+    char kbuf[sizeof("hibernate\n")];
+    nv_power_state_t power_state;
+    unsigned i;
+
+    if (!NV_IS_SUSER())
+    {
+        return -EPERM;
+    }
+
+    if (count < strlen("resume") || count > sizeof(kbuf))
+    {
+        return -EINVAL;
+    }
+
+    if (copy_from_user(kbuf, buf, count))
+    {
+        return -EFAULT;
+    }
+
+    count = min(count, sizeof(kbuf) - 1);
+    for (i = 0; i < count && isalpha(kbuf[i]); i++);
+    kbuf[i] = '\0';
+
+    if (strcasecmp(kbuf, "suspend") == 0)
+    {
+        power_state = NV_POWER_STATE_IN_STANDBY;
+    }
+    else if (strcasecmp(kbuf, "hibernate") == 0)
+    {
+        power_state = NV_POWER_STATE_IN_HIBERNATE;
+    }
+    else if (strcasecmp(kbuf, "resume") == 0)
+    {
+        power_state = NV_POWER_STATE_RUNNING;
+    }
+    else
+    {
+        return -EINVAL;
+    }
+
+    status = nv_set_system_power_state(power_state, nv_pm_action_depth);
+
+    return (status != NV_OK) ? -EIO : count;
+}
+
+static int
+nv_procfs_open_suspend(
+    struct inode *inode,
+    struct file *file
+)
+{
+    return single_open(file, nv_procfs_show_suspend, NULL);
+}
+
+static struct file_operations nv_procfs_suspend_fops = {
+    .owner   = THIS_MODULE,
+    .open    = nv_procfs_open_suspend,
+    .read    = seq_read,
+    .write   = nv_procfs_write_suspend,
+    .llseek  = seq_lseek,
+    .release = single_release
+};
+#endif
+
 /*
  * Forwards error to nv_log_error which exposes data to vendor callback
  */
@@ -482,9 +690,24 @@ nv_procfs_open_exercise_error_forwarding(
     int retval;
 
     retval = nv_procfs_open_file(inode, file, &nvfp);
+    if (retval < 0)
+    {
+        return retval;
+    }
 
-    if (retval == 0)
-        single_open(file, nv_procfs_show_exercise_error_forwarding, nvfp);
+    retval = single_open(file, nv_procfs_show_exercise_error_forwarding, nvfp);
+    if (retval < 0)
+    {
+        nv_procfs_close_file(nvfp);
+        return retval;
+    }
+
+    retval = NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE();
+    if (retval < 0)
+    {
+        single_release(inode, file);
+        nv_procfs_close_file(nvfp);
+    }
 
     return retval;
 }
@@ -497,7 +720,6 @@ nv_procfs_close_exercise_error_forwarding(
 {
     struct seq_file *s = file->private_data;
     nv_file_private_t *nvfp = s->private;
-    nvidia_stack_t *sp = nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
     nv_state_t *nv = nvfp->proc_data;
     char *proc_buffer = &((char *)nvfp->data)[0];
     size_t count = nvfp->off;
@@ -505,8 +727,6 @@ nv_procfs_close_exercise_error_forwarding(
     NvU32 xid = 0;
     const NvU8 MAX_XID_DIGITS = 3;
 
-    single_release(inode, file);
-
     while (i < count && i <= MAX_XID_DIGITS && proc_buffer[i] != ',')
     {
         if (proc_buffer[i] < '0' || proc_buffer[i] > '9')
@@ -525,16 +745,11 @@ nv_procfs_close_exercise_error_forwarding(
         status = -EINVAL;
 
 done:
-    if (NULL != nvfp->data)
-        os_free_mem(nvfp->data);
+    NV_READ_UNLOCK_SYSTEM_PM_LOCK();
 
-    if (sp != NULL)
-    {
-        nv_kmem_cache_free_stack(sp);
-        nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS] = NULL;
-    }
+    single_release(inode, file);
 
-    nv_free_file_private(nvfp);
+    nv_procfs_close_file(nvfp);
 
     return status;
 }
@@ -580,9 +795,23 @@ nv_procfs_open_unbind_lock(
     int retval;
 
     retval = nv_procfs_open_file(inode, file, &nvfp);
-    if (retval == 0)
+    if (retval < 0)
     {
-        single_open(file, nv_procfs_read_unbind_lock, nvfp);
+        return retval;
+    }
+
+    retval = single_open(file, nv_procfs_read_unbind_lock, nvfp);
+    if (retval < 0)
+    {
+        nv_procfs_close_file(nvfp);
+        return retval;
+    }
+
+    retval = NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE();
+    if (retval < 0)
+    {
+        single_release(inode, file);
+        nv_procfs_close_file(nvfp);
     }
 
     return retval;
@@ -595,18 +824,13 @@ nv_procfs_close_unbind_lock(
 )
 {
     struct seq_file *s = file->private_data;
-    nv_file_private_t *nvfp;
+    nv_file_private_t *nvfp = s->private;
     nv_state_t *nv;
-    nvidia_stack_t *sp = NULL;
+    nvidia_stack_t *sp = nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
     int rc = 0;
     nv_linux_state_t * nvl;
     int value;
 
-    nvfp = s->private;
-    single_release(inode, file);
-
-    sp = nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
-
     if (0 != nvfp->off)
     {
         nv = nvfp->proc_data;
@@ -632,42 +856,27 @@ nv_procfs_close_unbind_lock(
 
             if (nv->flags & NV_FLAG_UNBIND_LOCK)
             {
-                nv_printf(NV_DBG_ERRORS, "NVRM: UnbindLock acquired for device "
-                          "%04x:%02x:%02x.0\n",
-                          nv->pci_info.domain, nv->pci_info.bus,
-                          nv->pci_info.slot);
-
+                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock acquired");
             }
             else
             {
-                nv_printf(NV_DBG_ERRORS, "NVRM: Could not acquire UnbindLock for device "
-                          "%04x:%02x:%02x.0\n",
-                          nv->pci_info.domain, nv->pci_info.bus,
-                          nv->pci_info.slot);
+                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "Could not acquire UnbindLock");
             }
         }
         else if ((value == 0) && (nv->flags & NV_FLAG_UNBIND_LOCK))
         {
             nv->flags &= ~NV_FLAG_UNBIND_LOCK;
-            nv_printf(NV_DBG_ERRORS, "NVRM: UnbindLock released for device "
-                      "%04x:%02x:%02x.0\n",
-                      nv->pci_info.domain, nv->pci_info.bus,
-                      nv->pci_info.slot);
+            NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock released");
         }
         up(&nvl->ldata_lock);
     }
 
 done:
-    if (NULL != nvfp->data)
-        os_free_mem(nvfp->data);
+    NV_READ_UNLOCK_SYSTEM_PM_LOCK();
 
-    if (sp != NULL)
-    {
-        nv_kmem_cache_free_stack(sp);
-        nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS] = NULL;
-    }
+    single_release(inode, file);
 
-    nv_free_file_private(nvfp);
+    nv_procfs_close_file(nvfp);
 
     return rc;
 }
@@ -818,8 +1027,24 @@ nv_procfs_open_offline_pages(
     nv_file_private_t *nvfp = NULL;
 
     retval = nv_procfs_open_file(inode, file, &nvfp);
-    if (retval == 0)
-        single_open(file, nv_procfs_read_offline_pages, nvfp);
+    if (retval < 0)
+    {
+        return retval;
+    }
+
+    retval = single_open(file, nv_procfs_read_offline_pages, nvfp);
+    if (retval < 0)
+    {
+        nv_procfs_close_file(nvfp);
+        return retval;
+    }
+
+    retval = NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE();
+    if (retval < 0)
+    {
+        single_release(inode, file);
+        nv_procfs_close_file(nvfp);
+    }
 
     return retval;
 }
@@ -830,21 +1055,16 @@ nv_procfs_close_offline_pages(
         struct file  *file
 )
 {
-    int retval = 0;
     struct seq_file *s = file->private_data;
     nv_file_private_t *nvfp = s->private;
-    nvidia_stack_t *sp = nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
+
+    NV_READ_UNLOCK_SYSTEM_PM_LOCK();
 
     single_release(inode, file);
 
-    if (sp != NULL)
-    {
-        nv_kmem_cache_free_stack(sp);
-        nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS] = NULL;
-    }
+    nv_procfs_close_file(nvfp);
 
-    nv_free_file_private(nvfp);
-    return retval;
+    return 0;
 }
 
 static int
@@ -893,9 +1113,24 @@ nv_procfs_open_numa_status(
     nv_file_private_t *nvfp = NULL;
 
     retval = nv_procfs_open_file(inode, file, &nvfp);
+    if (retval < 0)
+    {
+        return retval;
+    }
 
-    if (retval == 0)
-        single_open(file, nv_procfs_read_numa_status, nvfp);
+    retval = single_open(file, nv_procfs_read_numa_status, nvfp);
+    if (retval < 0)
+    {
+        nv_procfs_close_file(nvfp);
+        return retval;
+    }
+
+    retval = NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE();
+    if (retval < 0)
+    {
+        single_release(inode, file);
+        nv_procfs_close_file(nvfp);
+    }
 
     return retval;
 }
@@ -917,7 +1152,6 @@ nv_procfs_close_numa_status(
     nv_numa_status_t current_state = nv_get_numa_status(nvl);
     char *cmd = nvfp->data;
 
-    single_release(inode, file);
     down(&nvl->ldata_lock);
 
     if (nvfp->off != 0)
@@ -974,16 +1208,12 @@ nv_procfs_close_numa_status(
 done:
     up(&nvl->ldata_lock);
 
-    if (nvfp->data != NULL)
-        os_free_mem(nvfp->data);
+    NV_READ_UNLOCK_SYSTEM_PM_LOCK();
 
-    if (sp != NULL)
-    {
-        nv_kmem_cache_free_stack(sp);
-        nvfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS] = NULL;
-    }
+    single_release(inode, file);
+
+    nv_procfs_close_file(nvfp);
 
-    nv_free_file_private(nvfp);
     return retval;
 }
 
@@ -1014,7 +1244,7 @@ nv_procfs_read_text_file(
     return 0;
 }
 
-NV_DEFINE_PROCFS_SINGLE_FILE(text_file);
+NV_DEFINE_SINGLE_NVRM_PROCFS_FILE(text_file);
 
 static void
 nv_procfs_add_text_file(
@@ -1085,6 +1315,16 @@ int nv_register_procfs(void)
     if (!entry)
         goto failed;
 
+#if defined(CONFIG_PM)
+    entry = NV_CREATE_PROC_FILE("suspend_depth", proc_nvidia, suspend_depth, NULL);
+    if (!entry)
+        goto failed;
+
+    entry = NV_CREATE_PROC_FILE("suspend", proc_nvidia, suspend, NULL);
+    if (!entry)
+        goto failed;
+#endif
+
     proc_nvidia_warnings = NV_CREATE_PROC_DIR("warnings", proc_nvidia);
     if (!proc_nvidia_warnings)
         goto failed;
@@ -1114,7 +1354,7 @@ int nv_register_procfs(void)
 #if defined(CONFIG_PROC_FS)
 failed:
     nv_procfs_unregister_all(proc_nvidia, proc_nvidia);
-    return -1;
+    return -ENOMEM;
 #endif
 }
 
@@ -1138,7 +1378,7 @@ int nv_procfs_add_gpu(nv_linux_state_t *nvl)
 
     snprintf(name, sizeof(name), "%04x:%02x:%02x.%1x",
                    nv->pci_info.domain, nv->pci_info.bus,
-                   nv->pci_info.slot, PCI_FUNC(nvl->dev->devfn));
+                   nv->pci_info.slot, PCI_FUNC(nvl->pci_dev->devfn));
 
     proc_nvidia_gpu = NV_CREATE_PROC_DIR(name, proc_nvidia_gpus);
     if (!proc_nvidia_gpu)
